"""Backward compatibility shim — moved to synix.build.transforms."""

from synix.build.transforms import (  # noqa: F401
    PROMPTS_DIR,
    BaseTransform,
    get_transform,
    register_transform,
)
